@@ -125,6 +125,7 @@ extern void OperClassValidatorDel(OperClassValidator *validator);
 
 extern ConfigItem_ban  *Find_ban_ip(aClient *sptr);
 extern void add_ListItem(ListStruct *, ListStruct **);
+extern void append_ListItem(ListStruct *item, ListStruct **list);
 extern void add_ListItemPrio(ListStructPrio *, ListStructPrio **, int);
 extern ListStruct *del_ListItem(ListStruct *, ListStruct **);
 extern aClient *find_match_server(char *mask);
@@ -504,6 +505,9 @@ extern time_t rfc2time(char *s);
 extern char *rfctime(time_t t, char *buf);
 extern void *MyMallocEx(size_t size);
 extern MODFUNC char  *ssl_get_cipher(SSL *ssl);
+extern SSLOptions *get_ssl_options_for_client(aClient *acptr);
+extern int outdated_tls_client(aClient *acptr);
+extern char *outdated_tls_client_build_string(char *pattern, aClient *acptr);
 extern long config_checkval(char *value, unsigned short flags);
 extern void config_status(char *format, ...) __attribute__((format(printf,1,2)));
 extern void init_random();
@@ -513,7 +517,6 @@ extern u_int32_t getrandom32();
 extern void ident_failed(aClient *cptr);
 
 extern MODVAR char extchmstr[4][64];
-extern MODVAR char extbanstr[EXTBANTABLESZ+1];
 
 extern int extcmode_default_requirechop(aClient *, aChannel *, char, char *, int, int);
 extern int extcmode_default_requirehalfop(aClient *, aChannel *, char, char *, int, int);
@@ -601,7 +604,6 @@ extern int del_dccallow(aClient *sptr, aClient *optr);
 extern void delete_linkblock(ConfigItem_link *link_ptr);
 extern void delete_classblock(ConfigItem_class *class_ptr);
 extern void del_async_connects(void);
-extern void make_extbanstr(void);
 extern void isupport_init(void);
 extern void clicap_init(void);
 extern int do_cmd(aClient *cptr, aClient *sptr, char *cmd, int parc, char *parv[]);
@@ -659,7 +661,7 @@ extern MODVAR void (*send_moddata_members)(aClient *srv);
 extern MODVAR void (*broadcast_moddata_client)(aClient *acptr);
 extern MODVAR int (*check_banned)(aClient *cptr);
 extern MODVAR void (*introduce_user)(aClient *to, aClient *acptr);
-extern MODVAR int (*check_deny_version)(aClient *cptr, char *version_string, int protocol, char *flags);
+extern MODVAR int (*check_deny_version)(aClient *cptr, char *software, int protocol, char *flags);
 extern MODVAR int (*match_user)(char *rmask, aClient *acptr, int options);
 extern MODVAR void (*userhost_save_current)(aClient *sptr);
 extern MODVAR void (*userhost_changed)(aClient *sptr);
@@ -672,6 +674,7 @@ extern MODVAR char *(*charsys_get_current_languages)(void);
 extern MODVAR aMotdFile opermotd, svsmotd, motd, botmotd, smotd, rules;
 extern MODVAR int max_connection_count;
 extern int add_listmode(Ban **list, aClient *cptr, aChannel *chptr, char *banid);
+extern int add_listmode_ex(Ban **list, aClient *cptr, aChannel *chptr, char *banid, char *setby, TS seton);
 extern int del_listmode(Ban **list, aChannel *chptr, char *banid);
 extern int Halfop_mode(long mode);
 extern char *clean_ban_mask(char *, int, aClient *);
@@ -683,7 +686,7 @@ extern MODVAR char *casemapping[2];
 extern MODVAR aTKline *tklines[TKLISTLEN];
 extern char *cmdname_by_spamftarget(int target);
 extern void unrealdns_delreq_bycptr(aClient *cptr);
-extern void sendtxtnumeric(aClient *to, char *pattern, ...) __attribute__((format(printf,2,3)));;
+extern void sendtxtnumeric(aClient *to, char *pattern, ...) __attribute__((format(printf,2,3)));
 extern void unrealdns_gethostbyname_link(char *name, ConfigItem_link *conf, int ipv4_only);
 extern void unrealdns_delasyncconnects(void);
 extern int is_autojoin_chan(char *chname);
@@ -759,7 +762,7 @@ extern int has_channel_mode(aChannel *chptr, char mode);
 extern int has_user_mode(aClient *acptr, char mode);
 extern long find_user_mode(char mode);
 extern void start_listeners(void);
-extern void buildvarstring(char *inbuf, char *outbuf, size_t len, char *name[], char *value[]);
+extern void buildvarstring(const char *inbuf, char *outbuf, size_t len, const char *name[], const char *value[]);
 extern void reinit_ssl(aClient *);
 extern int m_error(aClient *cptr, aClient *sptr, int parc, char *parv[]);
 extern int m_dns(aClient *cptr, aClient *sptr, int parc, char *parv[]);
@@ -783,9 +786,9 @@ extern int invisible_user_in_channel(aClient *target, aChannel *chptr);
 extern MODVAR int ssl_client_index;
 extern SSLOptions *FindSSLOptionsForUser(aClient *acptr);
 extern int IsWebsocket(aClient *acptr);
-extern PlaintextPolicy plaintextpolicy_strtoval(char *s);
-extern char *plaintextpolicy_valtostr(PlaintextPolicy policy);
-extern char plaintextpolicy_valtochar(PlaintextPolicy policy);
+extern Policy policy_strtoval(char *s);
+extern char *policy_valtostr(Policy policy);
+extern char policy_valtochar(Policy policy);
 extern int verify_certificate(SSL *ssl, char *hostname, char **errstr);
 extern char *certificate_name(SSL *ssl);
 extern int cipher_check(SSL_CTX *ctx, char **errstr);
@@ -798,3 +801,10 @@ extern char *spki_fingerprint(aClient *acptr);
 extern int is_module_loaded(char *name);
 extern void close_std_descriptors(void);
 extern int banned_client(aClient *acptr, char *bantype, char *reason, int global, int noexit);
+extern char *mystpcpy(char *dst, const char *src);
+extern size_t add_sjsby(char *buf, char *setby, TS seton);
+extern MaxTarget *findmaxtarget(char *cmd);
+extern void setmaxtargets(char *cmd, int limit);
+extern void freemaxtargets(void);
+extern int max_targets_for_command(char *cmd);
+extern void set_targmax_defaults(void);
