@@ -180,7 +180,10 @@ typedef OperPermission (*OperClassEntryEvalCallback)(OperClassACLEntryVar* varia
 #define	USERLEN		10
 #define	REALLEN	 	50
 #define SVIDLEN		30
-#define	TOPICLEN	307
+#define MAXTOPICLEN	360	/* absolute maximum permitted topic length (above this = potential desynch) */
+#define MAXAWAYLEN	360	/* absolute maximum permitted away length (above this = potential desynch) */
+#define MAXKICKLEN	360	/* absolute maximum kick length (above this = only cutoff danger) */
+#define MAXQUITLEN	395	/* absolute maximum quit length (above this = only cutoff danger) */
 #define	CHANNELLEN	32
 #define	PASSWDLEN 	48	/* was 20, then 32, now 48. */
 #define	KEYLEN		23
@@ -193,6 +196,7 @@ typedef OperPermission (*OperClassEntryEvalCallback)(OperClassACLEntryVar* varia
 #define SIDLEN           3
 #define SWHOISLEN	256
 #define UMODETABLESZ (sizeof(long) * 8)
+#define MAXCCUSERS		20 /* Maximum for set::anti-flood::target-limit::max-concurrent-conversations */
 /*
  * Watch it - Don't change this unless you also change the ERR_TOOMANYWATCH
  * and PROTOCOL_SUPPORTED settings.
@@ -360,7 +364,7 @@ typedef OperPermission (*OperClassEntryEvalCallback)(OperClassACLEntryVar* varia
 #define PROTO_EXTSWHOIS 0x800000	/* extended SWHOIS support */
 #define PROTO_CAP_CHGHOST	0x1000000	/* CAP chghost */
 #define PROTO_CAP_EXTENDED_JOIN	0x2000000	/* CAP extended-join */
-
+#define PROTO_SJSBY		0x4000000	/* SJOIN setby information (TS and nick) */
 /*
  * flags macros.
  */
@@ -460,6 +464,7 @@ typedef OperPermission (*OperClassEntryEvalCallback)(OperClassACLEntryVar* varia
 #define SupportUMODE2(x)	(CHECKPROTO(x, PROTO_UMODE2))
 #define SupportVL(x)		(CHECKPROTO(x, PROTO_VL))
 #define SupportSJ3(x)		(CHECKPROTO(x, PROTO_SJ3))
+#define SupportSJSBY(x)		(CHECKPROTO(x, PROTO_SJSBY))
 #define SupportVHP(x)		(CHECKPROTO(x, PROTO_VHP))
 #define SupportTKLEXT(x)	(CHECKPROTO(x, PROTO_TKLEXT))
 #define SupportTKLEXT2(x)	(CHECKPROTO(x, PROTO_TKLEXT2))
@@ -475,6 +480,7 @@ typedef OperPermission (*OperClassEntryEvalCallback)(OperClassACLEntryVar* varia
 #define SetUMODE2(x)		((x)->local->proto |= PROTO_UMODE2)
 #define SetVL(x)		((x)->local->proto |= PROTO_VL)
 #define SetSJ3(x)		((x)->local->proto |= PROTO_SJ3)
+#define SetSJSBY(x)		((x)->local->proto |= PROTO_SJSBY)
 #define SetVHP(x)		((x)->local->proto |= PROTO_VHP)
 #define SetTKLEXT(x)	((x)->local->proto |= PROTO_TKLEXT)
 #define SetTKLEXT2(x)	((x)->local->proto |= PROTO_TKLEXT2)
@@ -674,8 +680,12 @@ struct User {
 	struct {
 		time_t nick_t;
 		unsigned char nick_c;
-		time_t away_t;			/* last time the user set away */
+		time_t away_t;		/* last time the user set away */
 		unsigned char away_c;	/* number of times away has been set */
+		time_t knock_t;		/* last time the user has knocked */
+		unsigned char knock_c;	/* number of times the user knocked */
+		time_t invite_t;	/* last time the user used /invite */
+		unsigned char invite_c;	/* number of times the user used /invite */
 	} flood;
 	TS lastaway;
 };
@@ -698,6 +708,7 @@ struct Server {
 	struct {
 		char *chanmodes[4];
 		int protocol;
+		char *software;
 	} features;
 };
 
@@ -709,7 +720,7 @@ struct Server {
 #define M_ALIAS			0x0020
 #define M_RESETIDLE		0x0040
 #define M_VIRUS			0x0080
-#define M_ANNOUNCE		0x0100
+#define M_ANNOUNCE		0x0100 /* deprecated! */
 #define M_OPER			0x0200
 
 
@@ -862,7 +873,7 @@ struct LocalClient {
 	TS   last;		/* last time a RESETIDLE message was received */
 	TS   nexttarget;	/* next time that a new target will be allowed (msg/notice/invite) */
  	TS   nextnick;		/* Time the next nick change will be allowed */
-	u_char targets[MAXTARGETS];	/* hash values of targets */
+	u_char targets[MAXCCUSERS];	/* hash values of targets */
 	char buffer[BUFSIZE];	/* Incoming message buffer */
 	short lastsq;		/* # of 2k blocks when sendqueued called last */
 	dbuf sendQ;		/* Outgoing message queue--if socket full */
@@ -1118,6 +1129,11 @@ struct _configitem_oper {
 	int maxlogins;
 };
 
+/** The SSL options that are used in set::ssl and otherblocks::ssl-options.
+ * NOTE: If you add something here then you must also update the
+ *       conf_sslblock() function in s_conf.c to have it inherited
+ *       from set::ssl to the other config blocks!
+ */
 typedef struct _ssloptions SSLOptions;
 struct _ssloptions {
 	char *certificate_file;
@@ -1128,6 +1144,8 @@ struct _ssloptions {
 	char *ciphers;
 	char *ciphersuites;
 	char *ecdh_curves;
+	char *outdated_protocols;
+	char *outdated_ciphers;
 	long options;
 	int renegotiate_bytes;
 	int renegotiate_timeout;
@@ -1376,6 +1394,9 @@ struct _configitem_offchans {
 #define HM_IPV4 2
 #define HM_IPV6 3
 
+#define SETTER_NICK 0
+#define SETTER_NICK_USER_HOST 1
+
 /*
  * statistics structures
  */
@@ -1534,6 +1555,7 @@ struct DSlink {
 	} value;
 };
 #define AddListItem(item,list) add_ListItem((ListStruct *)item, (ListStruct **)&list)
+#define AppendListItem(item,list) append_ListItem((ListStruct *)item, (ListStruct **)&list)
 #define DelListItem(item,list) del_ListItem((ListStruct *)item, (ListStruct **)&list)
 
 #define AddListItemPrio(item,list,prio) add_ListItemPrio((ListStructPrio *)item, (ListStructPrio **)&list, prio)
@@ -1756,6 +1778,14 @@ struct ThrottlingBucket *find_throttling_bucket(aClient *);
 void add_throttling_bucket(aClient *);
 int throttle_can_connect(aClient *);
 
+typedef struct _maxtargets MaxTarget;
+struct _maxtargets {
+	MaxTarget *prev, *next;
+	char *cmd;
+	int limit;
+};
+#define MAXTARGETS_MAX	1000000 /* used for 'max' */
+
 #define VERIFY_OPERCOUNT(clnt,tag) { if (IRCstats.operators < 0) verify_opercount(clnt,tag); } while(0)
 
 #define MARK_AS_OFFICIAL_MODULE(modinf)	do { if (modinf && modinf->handle) ModuleSetOptions(modinfo->handle, MOD_OPT_OFFICIAL, 1);  } while(0)
@@ -1784,10 +1814,10 @@ int throttle_can_connect(aClient *);
 #define MATCH_USE_IDENT             0x0100
 
 typedef enum {
-	PLAINTEXT_POLICY_ALLOW=1,
-	PLAINTEXT_POLICY_WARN=2,
-	PLAINTEXT_POLICY_DENY=3
-} PlaintextPolicy;
+	POLICY_ALLOW=1,
+	POLICY_WARN=2,
+	POLICY_DENY=3
+} Policy;
 
 #define NO_EXIT_CLIENT	99
 
