@@ -332,15 +332,15 @@ int identical_ban(char *one, char *two)
 	return 0;
 }
 
-/*
- * add_listmode - Add a listmode (+beI) with the specified banid to
- *                the specified channel.
+/** Add a listmode (+beI) with the specified banid to
+ *  the specified channel. (Extended version with
+ *  set by nick and set on timestamp)
  */
-
-int add_listmode(Ban **list, aClient *cptr, aChannel *chptr, char *banid)
+int add_listmode_ex(Ban **list, aClient *cptr, aChannel *chptr, char *banid, char *setby, TS seton)
 {
 	Ban *ban;
 	int cnt = 0, len;
+	int do_not_add = 0;
 
 	if (MyClient(cptr))
 		(void)collapse(banid);
@@ -348,9 +348,13 @@ int add_listmode(Ban **list, aClient *cptr, aChannel *chptr, char *banid)
 	len = strlen(banid);
 	if (!*list && ((len > MAXBANLENGTH) || (MAXBANS < 1)))
 	{
-		sendto_one(cptr, err_str(ERR_BANLISTFULL),
-			me.name, cptr->name, chptr->chname, banid);
-		return -1;
+		if (MyClient(cptr))
+		{
+			/* Only send the error to local clients */
+			sendto_one(cptr, err_str(ERR_BANLISTFULL),
+				me.name, cptr->name, chptr->chname, banid);
+		}
+		do_not_add = 1;
 	}
 	for (ban = *list; ban; ban = ban->next)
 	{
@@ -366,19 +370,56 @@ int add_listmode(Ban **list, aClient *cptr, aChannel *chptr, char *banid)
 				sendto_one(cptr, err_str(ERR_BANLISTFULL),
 				    me.name, cptr->name, chptr->chname, banid);
 			}
-			return -1;
+			do_not_add = 1;
 		}
 		if (identical_ban(ban->banstr, banid))
-			return -1;
+			break; /* update existing ban (potentially) */
 	}
-	ban = make_ban();
-	ban->next = *list;
-	ban->banstr = strdup(banid);
-	ban->who = strdup(cptr->name);
-	ban->when = TStime();
-	*list = ban;
+
+	/* Create a new ban if needed */
+	if (!ban)
+	{
+		if (do_not_add)
+		{
+			/* The banlist is full and trying to add a new ban.
+			 * This is not permitted.
+			 */
+			return -1;
+		}
+		ban = make_ban();
+		ban->next = *list;
+		*list = ban;
+	}
+
+	if ((ban->when > 0) && (seton >= ban->when))
+	{
+		/* Trying to add the same ban while an older version
+		 * or identical version of the ban already exists.
+		 */
+		return -1;
+	}
+
+	/* Update/set if this ban is new or older than existing one */
+	safestrdup(ban->banstr, banid); /* cAsE may differ, use oldest version of it */
+	safestrdup(ban->who, setby);
+	ban->when = seton;
 	return 0;
 }
+
+/** Add a listmode (+beI) with the specified banid to
+ *  the specified channel. (Simplified version)
+ */
+int add_listmode(Ban **list, aClient *cptr, aChannel *chptr, char *banid)
+{
+	char *setby = cptr->name;
+	char nuhbuf[NICKLEN+USERLEN+HOSTLEN+4];
+
+	if (IsPerson(cptr) && (iConf.ban_setter == SETTER_NICK_USER_HOST))
+		setby = make_nick_user_host_r(nuhbuf, cptr->name, cptr->user->username, GetHost(cptr));
+
+	return add_listmode_ex(list, cptr, chptr, banid, setby, TStime());
+}
+
 /*
  * del_listmode - delete a listmode (+beI) from a channel
  *                that matches the specified banid.
@@ -419,7 +460,7 @@ int del_listmode(Ban **list, aChannel *chptr, char *banid)
  */
 inline Ban *is_banned(aClient *sptr, aChannel *chptr, int type)
 {
-	return is_banned_with_nick(sptr, chptr, type, sptr->name);
+	return is_banned_with_nick(sptr, chptr, type, NULL);
 }
 
 /** ban_check_mask - Checks if the user matches the specified n!u@h mask -or- run an extended ban.
@@ -459,52 +500,63 @@ inline int ban_check_mask(aClient *sptr, aChannel *chptr, char *banstr, int type
  * @param sptr   Client to check (can be remote client)
  * @param chptr  Channel to check
  * @param type   Type of ban to check for (BANCHK_*)
- * @param nick   Nick of the user
+ * @param nick   Nick of the user (or NULL, to default to sptr->name)
  * @returns      A pointer to the ban struct if banned, otherwise NULL.
  */
 Ban *is_banned_with_nick(aClient *sptr, aChannel *chptr, int type, char *nick)
 {
-	Ban *tmp, *tmp2;
+	Ban *ban, *ex;
+	char savednick[NICKLEN+1];
+
+	/* It's not really doable to pass 'nick' to all the ban layers,
+	 * including extbans (with stacking) and so on. Or at least not
+	 * without breaking several module API's.
+	 * So, instead, we temporarily set 'sptr->name' to 'nick' and
+	 * restore it to the orginal value at the end of this function.
+	 * This is possible because all these layers never send a message
+	 * to 'sptr' and only indicate success/failure.
+	 * Note that all this ONLY happens if is_banned_with_nick() is called
+	 * with a non-NULL nick. That doesn't happen much. In UnrealIRCd
+	 * only in case of '/NICK newnick'. This fixes #5165.
+	 */
+	if (nick)
+	{
+		strlcpy(savednick, sptr->name, sizeof(savednick));
+		strlcpy(sptr->name, nick, sizeof(sptr->name));
+	}
 
 	/* We check +b first, if a +b is found we then see if there is a +e.
 	 * If a +e was found we return NULL, if not, we return the ban.
 	 */
-	for (tmp = chptr->banlist; tmp; tmp = tmp->next)
-	{
-		if (!ban_check_mask(sptr, chptr, tmp->banstr, type, 0))
-			continue;
 
-		/* Ban found, now check for +e */
-		for (tmp2 = chptr->exlist; tmp2; tmp2 = tmp2->next)
-		{
-			if (ban_check_mask(sptr, chptr, tmp2->banstr, type, 0))
-				return NULL; /* except matched */
-		}
-		break; /* ban found and not on except */
+	for (ban = chptr->banlist; ban; ban = ban->next)
+	{
+		if (ban_check_mask(sptr, chptr, ban->banstr, type, 0))
+			break;
 	}
 
-	return (tmp);
-}
-
-/*
- * Checks if the "user" IRC is banned, used by +mu.
- */
-static int is_irc_banned(aChannel *chptr)
-{
-	Ban *tmp;
-	/* Check for this user, ident/host are "illegal" on purpose */
-	char *check = "IRC!\001@\001";
-	
-	for (tmp = chptr->banlist; tmp; tmp = tmp->next)
-		if (match(tmp->banstr, check) == 0)
+	if (ban)
+	{
+		/* Ban found, now check for +e */
+		for (ex = chptr->exlist; ex; ex = ex->next)
 		{
-			/* Ban found, now check for +e */
-			for (tmp = chptr->exlist; tmp; tmp = tmp->next)
-				if (match(tmp->banstr, check) == 0)
-					return 0; /* In exception list */
-			return 1;
+			if (ban_check_mask(sptr, chptr, ex->banstr, type, 0))
+			{
+				/* except matched */
+				ban = NULL;
+				break;
+			}
 		}
-	return 0;
+		/* user is not on except, 'ban' stays non-NULL. */
+	}
+
+	if (nick)
+	{
+		/* Restore the nick */
+		strlcpy(sptr->name, savednick, sizeof(sptr->name));
+	}
+
+	return ban;
 }
 
 /*
@@ -709,7 +761,7 @@ int  can_send(aClient *cptr, aChannel *chptr, char *msgtext, int notice)
 	}
 
 	lp = find_membership_link(cptr->user->channel, chptr);
-	if (chptr->mode.mode & MODE_MODERATED && !op_can_override("override:message:moderated",cptr,chptr,NULL) &&
+	if (chptr->mode.mode & MODE_MODERATED && !op_can_override("channel:override:message:moderated",cptr,chptr,NULL) &&
 	    (!lp
 	    || !(lp->flags & (CHFL_CHANOP | CHFL_VOICE | CHFL_CHANOWNER |
 	CHFL_HALFOP | CHFL_CHANPROT))))
@@ -739,7 +791,7 @@ int  can_send(aClient *cptr, aChannel *chptr, char *msgtext, int notice)
 		return i;
 
 	/* Makes opers able to talk thru bans -Stskeeps suggested by The_Cat */
-	if (op_can_override("override:message:ban",cptr,chptr,NULL))
+	if (op_can_override("channel:override:message:ban",cptr,chptr,NULL))
 		return 0;
 
 	if ((!lp
@@ -963,7 +1015,7 @@ char *clean_ban_mask(char *mask, int what, aClient *cptr)
 	/* Extended ban? */
 	if ((*mask == '~') && mask[1] && (mask[2] == ':'))
 	{
-		if (RESTRICT_EXTENDEDBANS && MyClient(cptr) && !ValidatePermissionsForPath("channel:extbans",cptr,NULL,NULL,NULL))
+		if (RESTRICT_EXTENDEDBANS && MyClient(cptr) && !ValidatePermissionsForPath("immune:restrict-extendedbans",cptr,NULL,NULL,NULL))
 		{
 			if (!strcmp(RESTRICT_EXTENDEDBANS, "*"))
 			{
