@@ -1137,7 +1137,7 @@ int InitUnrealIRCd(int argc, char *argv[])
 			  type = Auth_FindType(NULL, p);
 			  if (type == -1)
 			  {
-			      type = AUTHTYPE_BCRYPT;
+			      type = AUTHTYPE_ARGON2;
 			  } else {
 			      p = *++argv;
 			      argc--;
@@ -1155,11 +1155,11 @@ int InitUnrealIRCd(int argc, char *argv[])
 			  {
 			      /* Hmmm.. is this warning really still true (and always) ?? */
 			      printf("WARNING: Password truncated to 8 characters due to 'crypt' algorithm. "
-		                 "You are suggested to use the 'bcrypt' algorithm instead.");
+		                 "You are suggested to use the 'argon2' algorithm instead.");
 				  p[8] = '\0';
 			  }
 			  if (!(result = Auth_Make(type, p))) {
-				  printf("Authentication failed\n");
+				  printf("Failed to generate password. Deprecated method? Try 'argon2' instead.\n");
 				  exit(0);
 			  }
 			  printf("Encrypted password is: %s\n", result);
@@ -1348,8 +1348,6 @@ int InitUnrealIRCd(int argc, char *argv[])
 	load_tunefile();
 	make_umodestr();
 	extcmodes_check_for_changes();
-	make_extbanstr();
-	isupport_init();
 	clicap_init();
 	if (!find_Command_simple("AWAY") /*|| !find_Command_simple("KILL") ||
 		!find_Command_simple("OPER") || !find_Command_simple("PING")*/)
