@@ -144,6 +144,7 @@ aClient *make_client(aClient *from, aClient *servr)
 	if (!from)
 	{
 		/* Local client */
+		const char *id;
 		
 		cptr->local = MyMallocEx(sizeof(aLocalClient));
 		
@@ -161,6 +162,11 @@ aClient *make_client(aClient *from, aClient *servr)
 
 		dbuf_queue_init(&cptr->local->recvQ);
 		dbuf_queue_init(&cptr->local->sendQ);
+
+		while (hash_find_id((id = uid_get()), NULL) != NULL)
+			;
+		strlcpy(cptr->id, id, sizeof cptr->id);
+		add_to_id_hash_table(cptr->id, cptr);
 	} else {
 		cptr->fd = -256;
 	}
@@ -189,6 +195,14 @@ void free_client(aClient *cptr)
 			
 			MyFree(cptr->local);
 		}
+		if (*cptr->id)
+		{
+			/* This is already del'd in exit_one_client, so we
+			 * only have it here in case a shortcut was taken,
+			 * such as from add_connection() to free_client().
+			 */
+			del_from_id_hash_table(cptr->id, cptr);
+		}
 	}
 	
 	safefree(cptr->ip);
@@ -257,6 +271,14 @@ aServer *make_server(aClient *cptr)
 		serv->up = NULL;
 		cptr->serv = serv;
 	}
+	if (strlen(cptr->id) > 3)
+	{
+		/* Probably the auto-generated UID for a server that
+		 * still uses the old protocol (without SID).
+		 */
+		del_from_id_hash_table(cptr->id, cptr);
+		*cptr->id = '\0';
+	}
 	return cptr->serv;
 }
 
@@ -345,6 +367,7 @@ void remove_client_from_list(aClient *cptr)
 		safefree(cptr->serv->features.chanmodes[1]);
 		safefree(cptr->serv->features.chanmodes[2]);
 		safefree(cptr->serv->features.chanmodes[3]);
+		safefree(cptr->serv->features.software);
 		MyFree(cptr->serv);
 #ifdef	DEBUGMODE
 		servs.inuse--;
@@ -538,6 +561,23 @@ void add_ListItem(ListStruct *item, ListStruct **list) {
 	*list = item;
 }
 
+/* (note that if you end up using this, you should probably
+ *  use a circular linked list instead)
+ */
+void append_ListItem(ListStruct *item, ListStruct **list) {
+	ListStruct *l;
+
+	if (!*list)
+	{
+		*list = item;
+		return;
+	}
+
+	for (l = *list; l->next; l = l->next);
+	l->next = item;
+	item->prev = l;
+}
+
 ListStruct *del_ListItem(ListStruct *item, ListStruct **list) {
 	ListStruct *l, *ret;
 
