@@ -31,7 +31,7 @@ int _verify_link(aClient *cptr, aClient *sptr, char *servername, ConfigItem_link
 void _send_protoctl_servers(aClient *sptr, int response);
 void _send_server_message(aClient *sptr);
 void _introduce_user(aClient *to, aClient *acptr);
-int _check_deny_version(aClient *cptr, char *version_string, int protocol, char *flags);
+int _check_deny_version(aClient *cptr, char *software, int protocol, char *flags);
 
 static char buf[BUFSIZE];
 
@@ -83,7 +83,7 @@ int m_server_synch(aClient *cptr, ConfigItem_link *conf);
 /** Check deny version { } blocks.
  * NOTE: cptr will always be valid, but all the other values may be NULL or 0 !!!
  */
-int _check_deny_version(aClient *cptr, char *version_string, int protocol, char *flags)
+int _check_deny_version(aClient *cptr, char *software, int protocol, char *flags)
 {
 	ConfigItem_deny_version *vlines;
 	
@@ -400,12 +400,18 @@ skip_host_check:
 				get_client_name(cptr, TRUE));
 		return exit_client(cptr, cptr, &me, "Full class");
 	}
-	if (!IsLocal(cptr) && (iConf.plaintext_policy_server == PLAINTEXT_POLICY_DENY) && !IsSecure(cptr))
+	if (!IsLocal(cptr) && (iConf.plaintext_policy_server == POLICY_DENY) && !IsSecure(cptr))
 	{
 		sendto_one(cptr, "ERROR :Servers need to use SSL/TLS (set::plaintext-policy::server is 'deny')");
 		sendto_ops_and_log("Rejected insecure server %s. See https://www.unrealircd.org/docs/FAQ#ERROR:_Servers_need_to_use_SSL.2FTLS", cptr->name);
 		return exit_client(cptr, sptr, &me, "Servers need to use SSL/TLS (set::plaintext-policy::server is 'deny')");
 	}
+	if (IsSecure(cptr) && (iConf.outdated_tls_policy_server == POLICY_DENY) && outdated_tls_client(cptr))
+	{
+		sendto_one(cptr, "ERROR :Server is using an outdated SSL/TLS protocol or cipher (set::outdated-tls-policy::server is 'deny')");
+		sendto_ops_and_log("Rejected server %s using outdated %s. See https://www.unrealircd.org/docs/FAQ#server-outdated-tls", ssl_get_cipher(cptr->local->ssl), cptr->name);
+		return exit_client(cptr, sptr, &me, "Server using outdates SSL/TLS protocol or cipher (set::outdated-tls-policy::server is 'deny')");
+	}
 	if (link_out)
 		*link_out = link;
 	return 0;
@@ -845,7 +851,18 @@ int	m_server_synch(aClient *cptr, ConfigItem_link *aconf)
 	list_move(&cptr->lclient_node, &lclient_list);
 	list_add(&cptr->special_node, &server_list);
 	if ((Find_uline(cptr->name)))
+	{
+		if (cptr->serv && cptr->serv->features.software && !strncmp(cptr->serv->features.software, "UnrealIRCd-", 11))
+		{
+			sendto_realops("\002WARNING:\002 Bad ulines! It seems your server is misconfigured: "
+			               "your ulines { } block is matching an UnrealIRCd server (%s). "
+			               "This is not correct and will cause security issues. "
+			               "ULines should only be added for services! "
+			               "See https://www.unrealircd.org/docs/FAQ#bad-ulines",
+			               cptr->name);
+		}
 		cptr->flags |= FLAGS_ULINE;
+	}
 	(void)find_or_add(cptr->name);
 	if (IsSecure(cptr))
 	{
@@ -867,10 +884,15 @@ int	m_server_synch(aClient *cptr, ConfigItem_link *aconf)
 		 * Yeah.. there are still other cases when non-SSL links are fine (eg: local IP
 		 * of the same machine), we won't bother with detecting that. -- Syzop
 		 */
-		if (!IsLocal(cptr) && (iConf.plaintext_policy_server == PLAINTEXT_POLICY_WARN))
+		if (!IsLocal(cptr) && (iConf.plaintext_policy_server == POLICY_WARN))
 		{
 			sendto_realops("\002WARNING:\002 This link is unencrypted (non-SSL). We highly recommend to use "
-						   "SSL server linking. See https://www.unrealircd.org/docs/Linking_servers");
+			               "SSL/TLS for server linking. See https://www.unrealircd.org/docs/Linking_servers");
+		}
+		if (IsSecure(cptr) && (iConf.outdated_tls_policy_server == POLICY_WARN) && outdated_tls_client(cptr))
+		{
+			sendto_realops("\002WARNING:\002 This link is using an outdated SSL/TLS protocol or cipher (%s).",
+			               ssl_get_cipher(cptr->local->ssl));
 		}
 	}
 	(void)add_to_client_hash_table(cptr->name, cptr);
@@ -1395,16 +1417,6 @@ void send_channel_modes_sjoin(aClient *cptr, aChannel *chptr)
 	return;
 }
 
-char *mystpcpy(char *dst, const char *src)
-{
-	for (; *src; src++)
-		*dst++ = *src;
-	*dst = '\0';
-	return dst;
-}
-
-
-
 /** This will send "cptr" a full list of the modes for channel chptr,
  *
  * Half of it recoded by Syzop: the whole buffering and size checking stuff
@@ -1521,6 +1533,8 @@ void send_channel_modes_sjoin3(aClient *cptr, aChannel *chptr)
 	for (ban = chptr->banlist; ban; ban = ban->next)
 	{
 		p = tbuf;
+		if (SupportSJSBY(cptr))
+			p += add_sjsby(p, ban->who, ban->when);
 		*p++ = '&';
 		p = mystpcpy(p, ban->banstr);
 		*p++ = ' ';
@@ -1542,6 +1556,8 @@ void send_channel_modes_sjoin3(aClient *cptr, aChannel *chptr)
 	for (ban = chptr->exlist; ban; ban = ban->next)
 	{
 		p = tbuf;
+		if (SupportSJSBY(cptr))
+			p += add_sjsby(p, ban->who, ban->when);
 		*p++ = '"';
 		p = mystpcpy(p, ban->banstr);
 		*p++ = ' ';
@@ -1563,6 +1579,8 @@ void send_channel_modes_sjoin3(aClient *cptr, aChannel *chptr)
 	for (ban = chptr->invexlist; ban; ban = ban->next)
 	{
 		p = tbuf;
+		if (SupportSJSBY(cptr))
+			p += add_sjsby(p, ban->who, ban->when);
 		*p++ = '\'';
 		p = mystpcpy(p, ban->banstr);
 		*p++ = ' ';
