@@ -370,7 +370,7 @@ CMD_FUNC(m_stats)
 	/* Decide if we are looking for 1 char or a string */
 	if (parv[1][0] && !parv[1][1])
 	{
-		if (!ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL) && stats_operonly_short(parv[1][0]))
+		if (!ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL) && stats_operonly_short(parv[1][0]))
 		{
 			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, sptr->name);	
 			return 0;
@@ -380,7 +380,7 @@ CMD_FUNC(m_stats)
 	}
 	else
 	{
-		if (!ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL) && stats_operonly_long(parv[1]))
+		if (!ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL) && stats_operonly_long(parv[1]))
 		{
 			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, sptr->name);	
 			return 0;
@@ -393,7 +393,7 @@ CMD_FUNC(m_stats)
 		/* It was a short flag, so check oper only on long flags */
 		if (!parv[1][1])
 		{
-			if (!ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL) && stats_operonly_long(stat->longflag))
+			if (!ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL) && stats_operonly_long(stat->longflag))
 			{
 				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, sptr->name);
 				return 0;
@@ -402,7 +402,7 @@ CMD_FUNC(m_stats)
 		/* It was a long flag, so check oper only on short flags */
 		else
 		{
-			if (!ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL) && stats_operonly_short(stat->flag))
+			if (!ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL) && stats_operonly_short(stat->flag))
 			{
 				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, sptr->name);	
 				return 0;
@@ -624,7 +624,7 @@ int stats_port(aClient *sptr, char *para)
 	{
 		if (!(listener->options & LISTENER_BOUND))
 			continue;
-		if ((listener->options & LISTENER_SERVERSONLY) && !ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL))
+		if ((listener->options & LISTENER_SERVERSONLY) && !ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL))
 			continue;
 		sendto_one(sptr, ":%s NOTICE %s :*** Listener on %s:%i (%s): has %i client(s), options: %s %s",
 		           me.name, sptr->name,
@@ -826,7 +826,7 @@ int stats_mem(aClient *sptr, char *para)
 	     rm = 0,		/* res memory used */
 	     totcl = 0, totch = 0, totww = 0, tot = 0;
 
-	if (!ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL))
+	if (!ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL))
 	{
 		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, sptr->name);
 		return 0;
@@ -1146,7 +1146,7 @@ int stats_set(aClient *sptr, char *para)
 {
 	char *uhallow;
 
-	if (!ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL))
+	if (!ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL))
 	{
 		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, sptr->name);
 		return 0;
@@ -1193,7 +1193,7 @@ int stats_set(aClient *sptr, char *para)
 	sendto_one(sptr, ":%s %i %s :modes-on-join: %s %s", me.name, RPL_TEXT,
 		sptr->name, modebuf, parabuf);
 	sendto_one(sptr, ":%s %i %s :nick-length: %i", me.name, RPL_TEXT,
-		sptr->name, iConf.nicklen);
+		sptr->name, iConf.nick_length);
 	sendto_one(sptr, ":%s %i %s :snomask-on-oper: %s", me.name, RPL_TEXT,
 	    sptr->name, OPER_SNOMASK);
 	sendto_one(sptr, ":%s %i %s :snomask-on-connect: %s", me.name, RPL_TEXT,
@@ -1320,11 +1320,17 @@ int stats_set(aClient *sptr, char *para)
 	sendto_one(sptr, ":%s %i %s :check-target-nick-bans: %s", me.name, RPL_TEXT,
 		sptr->name, CHECK_TARGET_NICK_BANS ? "yes" : "no");
 	sendto_one(sptr, ":%s %i %s :plaintext-policy::user: %s", me.name, RPL_TEXT,
-		sptr->name, plaintextpolicy_valtostr(iConf.plaintext_policy_user));
+		sptr->name, policy_valtostr(iConf.plaintext_policy_user));
 	sendto_one(sptr, ":%s %i %s :plaintext-policy::oper: %s", me.name, RPL_TEXT,
-		sptr->name, plaintextpolicy_valtostr(iConf.plaintext_policy_oper));
+		sptr->name, policy_valtostr(iConf.plaintext_policy_oper));
 	sendto_one(sptr, ":%s %i %s :plaintext-policy::server: %s", me.name, RPL_TEXT,
-		sptr->name, plaintextpolicy_valtostr(iConf.plaintext_policy_server));
+		sptr->name, policy_valtostr(iConf.plaintext_policy_server));
+	sendto_one(sptr, ":%s %i %s :outdated-tls-policy::user: %s", me.name, RPL_TEXT,
+		sptr->name, policy_valtostr(iConf.outdated_tls_policy_user));
+	sendto_one(sptr, ":%s %i %s :outdated-tls-policy::oper: %s", me.name, RPL_TEXT,
+		sptr->name, policy_valtostr(iConf.outdated_tls_policy_oper));
+	sendto_one(sptr, ":%s %i %s :outdated-tls-policy::server: %s", me.name, RPL_TEXT,
+		sptr->name, policy_valtostr(iConf.outdated_tls_policy_server));
 	RunHook2(HOOKTYPE_STATS, sptr, "S");
 	return 1;
 }
@@ -1425,7 +1431,7 @@ int stats_linkinfoint(aClient *sptr, char *para, int all)
 	int remote = 0;
 	int wilds = 0;
 	int doall = 0;
-	int showports = ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL);
+	int showports = ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL);
 	int i;
 	aClient *acptr;
 	/*
@@ -1455,7 +1461,6 @@ int stats_linkinfoint(aClient *sptr, char *para, int all)
 	list_for_each_entry(acptr, &lclient_list, lclient_node)
 	{
 		if (IsInvisible(acptr) && (doall || wilds) &&
-			!ValidatePermissionsForPath("stats:viewinvisible",sptr,NULL,NULL,NULL) &&
 			!IsOper(acptr) && (acptr != sptr))
 			continue;
 		if (remote && doall && !IsServer(acptr) && !IsMe(acptr))
@@ -1473,7 +1478,7 @@ int stats_linkinfoint(aClient *sptr, char *para, int all)
 		ircsnprintf(pbuf, sizeof(pbuf), "%ld :%ld", (long)acptr->local->cputime,
 		      (long)(acptr->user && MyConnect(acptr)) ? TStime() - acptr->local->last : 0);
 #endif
-		if (ValidatePermissionsForPath("server:info",sptr,NULL,NULL,NULL))
+		if (ValidatePermissionsForPath("server:info:stats",sptr,NULL,NULL,NULL))
 		{
 			sendto_one(sptr, Lformat, me.name,
 				RPL_STATSLINKINFO, sptr->name, 
@@ -1492,10 +1497,6 @@ int stats_linkinfoint(aClient *sptr, char *para, int all)
 #else
 				pbuf);
 #endif
-			if (!IsServer(acptr) && !IsMe(acptr) && ValidatePermissionsForPath("privacy",acptr,NULL,NULL,NULL) && sptr != acptr)
-				sendto_one(acptr,
-					":%s NOTICE %s :*** %s did a /stats L on you! IP may have been shown",
-					me.name, acptr->name, sptr->name);
 		}
 		else if (!strchr(acptr->name, '.'))
 			sendto_one(sptr, Lformat, me.name,
